
package com.edulib.muse.proxy.samples;

import java.io.UnsupportedEncodingException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.LinkedHashMap;
import java.util.Map;
import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;

public class MuseProxyHMAC {

  public String generatedURL = null;

  public String generatedHmacDigest = null;

  public String museProxyURL = null;

  public String applicationID = null;

  public String secret = null;

  public String algorithm = null;

  public String separator = null;

  /**
   * Constructor is used to initialize parameters and generate hmacDigest and HMAC authentication URL.
   * 
   * 
   * @param museProxyURL represent the MuseProxy base URL.
   * @param applicationID represents the name of MuseProxy application for who want to generate hmac URL. 
   * @param secret represent a string, and the value must be the same with value configurate in configuration file for HMAC authentication.
   * @param algorithm represent the name of algorithm. Supported algorithms are: HmacMD5, HmacSHA1, HmacSHA224, HmacSHA256, HmacSHA384, HmacSHA512;
   * @param separator represent a string and the value must be the same with value configurate in configuration file.
   * @param parametersOrder is a List<String> with parameter order and this order must be the same with order from configuration file inside the tag <USED_PARAMS>.
   * @param request it is a object of type HttpServletRequest.
   */
  public MuseProxyHMAC(String museProxyURL, String applicationID, String secret, String algorithm, String separator, LinkedHashMap<String, String> parameters) {
    this.museProxyURL = museProxyURL;
    this.applicationID = applicationID;
    this.secret = secret;
    this.algorithm = algorithm;
    this.separator = separator;

    String value = getValueForGenerateDigest(parameters, separator);

    String parametersToSend = getParametersToCreateURL(parameters);

    this.generatedHmacDigest = getHMACDigest(algorithm, value, secret);

    this.generatedURL = getHmacURL(museProxyURL, applicationID, generatedHmacDigest, parametersToSend);

  }

  /**
   * 
   * @param customParameters represent the name of parameters and values from URL.
   * @return a String with parameterss and values who is used to create the HMAC URL.
   */
  public String getParametersToCreateURL(Map<String, String> customParameters) {
    String parametersToSend = "";
    for (Map.Entry<String, String> entry : customParameters.entrySet()) {
      String key = entry.getKey();
      if (!key.equals("userAddress") && !key.equals("userAgent") && !key.equals("referer")) {
        parametersToSend += "&" + key + "=" + entry.getValue();
      }
    }
    return parametersToSend;
  }

  /**
   * 
   * @param museProxyURL represent MuseProxy server base URL.
   * @param applicationID represents application ID that needs to be authenticated.
   * @param generatedHmacDigest represents the value obtained by calling method getHMACDigest(String algorithm, String value, String secret);
   * @param parametersToSend represents value obtained by calling method getParametersToCreateURL(Map<String, String> customParameters);
   * @return finalURL used for authentication.
   */
  public String getHmacURL(String museProxyURL, String applicationID, String generatedHmacDigest, String parametersToSend) {
    return museProxyURL + "/" + applicationID + "?sig=" + generatedHmacDigest + parametersToSend;
  }

  /**
   * 
   * @param customParameters represent a Map with parameters from URL.
   * @param separator represent a string who is used to separate values from customParameters.
   * @return value used for encoding.
   */
  public String getValueForGenerateDigest(Map<String, String> customParameters, String separator) {
    String value = "";
    int length = customParameters.size();
    for (int i = 0; i < length; i++) {
      if (i < length - 1) {
        value += customParameters.values().toArray()[i].toString() + separator;
      } else {
        value += customParameters.values().toArray()[i].toString();
      }
    }
    return value;
  }

  /**
   * 
   * @param algorithm represent name of algorithm used for enconding. The name of algorithm must to be the same with name configured in configuration file.
   * Supported algorithms is: HmacMD5, HmacSHA1, HmacSHA224, HmacSHA256, HmacSHA384, HmacSHA512;
   * @param value represent the value who is encoded
   * @param secret which is key to encrypt the message, this value must be same with value from ProxyLoginModuleHMAC.xml file
   * @return hmac digest obtained after encryption with specificated algorithm.
   * 
   */
  public String getHMACDigest(String algorithm, String value, String secret) {
    String digest = null;
    try {
      SecretKeySpec key = new SecretKeySpec((secret).getBytes("UTF-8"), algorithm);
      Mac mac = Mac.getInstance(algorithm);
      mac.init(key);
      byte[] bytes = mac.doFinal(value.getBytes("ASCII"));
      StringBuffer hash = new StringBuffer();
      for (int i = 0; i < bytes.length; i++) {
        String hex = Integer.toHexString(0xFF & bytes[i]);
        if (hex.length() == 1) {
          hash.append('0');
        }
        hash.append(hex);
      }
      digest = hash.toString();
    } catch (UnsupportedEncodingException e) {
    } catch (InvalidKeyException e) {
    } catch (NoSuchAlgorithmException e) {
    }
    return digest;
  }

}
